// author: Kwang-Hyun Park (akaii@kw.ac.kr)

#include "hamster.h"

#pragma comment(lib, "hamster.lib")

void example01(void) { // simple movement
	hamster_create();

	// move forward
	hamster_write(HAMSTER_LEFT_WHEEL, 50);
	hamster_write(HAMSTER_RIGHT_WHEEL, 50);
	hamster_wait(500);

	// move backward
	hamster_write(HAMSTER_LEFT_WHEEL, -50);
	hamster_write(HAMSTER_RIGHT_WHEEL, -50);
	hamster_wait(500);

	// stop
	hamster_write(HAMSTER_LEFT_WHEEL, 0);
	hamster_write(HAMSTER_RIGHT_WHEEL, 0);

	// don't forget to dispose
	hamster_dispose_all();
}

void example02(void) { // acceleration
	int i;

	hamster_create();

	// acceleration
	for(i = 0; i < 50; ++i) {
		hamster_write(HAMSTER_LEFT_WHEEL, i);
		hamster_write(HAMSTER_RIGHT_WHEEL, i);
		hamster_wait(20);
	}

	hamster_wait(500);

	// deceleration
	for(i = 50; i >= 0; --i) {
		hamster_write(HAMSTER_LEFT_WHEEL, i);
		hamster_write(HAMSTER_RIGHT_WHEEL, i);
		hamster_wait(20);
	}

	// don't forget to dispose
	hamster_dispose_all();
}

void example03(void) { // led
	hamster_create();

	hamster_write(HAMSTER_LEFT_LED, HAMSTER_LED_RED);
	hamster_write(HAMSTER_RIGHT_LED, HAMSTER_LED_GREEN);
	hamster_wait(1000);

	hamster_write(HAMSTER_LEFT_LED, HAMSTER_LED_OFF);
	hamster_write(HAMSTER_RIGHT_LED, HAMSTER_LED_OFF);

	// don't forget to dispose
	hamster_dispose_all();
}

void example04(void) { // basic movement
	int i;

	hamster_create();

	for(i = 0; i < 10; ++i) {
		hamster_write(HAMSTER_LEFT_WHEEL, 50);
		hamster_write(HAMSTER_RIGHT_WHEEL, 50);
		hamster_wait(500);

		hamster_write(HAMSTER_LEFT_WHEEL, -50);
		hamster_write(HAMSTER_RIGHT_WHEEL, -50);
		hamster_wait(500);

		hamster_write(HAMSTER_LEFT_WHEEL, -50);
		hamster_write(HAMSTER_RIGHT_WHEEL, 50);
		hamster_wait(500);
	}

	hamster_dispose_all();
}

void example05(void) { // staggering walk
	int count = 10;
	int accX;

	hamster_create();

	while(count > 0) {
		hamster_write(HAMSTER_LEFT_WHEEL, -100);
		hamster_write(HAMSTER_RIGHT_WHEEL, -100);
		accX = hamster_read(HAMSTER_ACCELERATION);
		if(accX > 2000 || accX < -2000) {
			hamster_write(HAMSTER_LEFT_WHEEL, 100);
			hamster_write(HAMSTER_RIGHT_WHEEL, 100);
			hamster_wait(250);

			hamster_write(HAMSTER_LEFT_WHEEL, -50);
			hamster_write(HAMSTER_RIGHT_WHEEL, 50);
			hamster_wait(500);

			--count;
		}
	}

	hamster_dispose_all();
}

void example06(void) { // siren
	int count = 0, buzzer, i;

	hamster_create();

	for(i = 0; i < 200; ++i) {
		buzzer = count + 500;
		count += 10;
		if(count > 200) count = 0;
		hamster_write(HAMSTER_BUZZER, buzzer);
		hamster_wait(20);
	}

	hamster_dispose_all();
}

double note = 0;
void theremin_execute(int hamster_index) {
	int proximity = hamster_read(HAMSTER_LEFT_PROXIMITY);
	if(proximity < 10) proximity = 0;
	note = (note * 9 + proximity) / 10;
	hamster_write(HAMSTER_NOTE, (int)note);
}

void example07(void) { // theremin sound
	note = 0;

	hamster_create();
	hamster_set_executable(theremin_execute);

	// 10 seconds
	hamster_wait(10000);

	hamster_dispose_all();
}

void hand_execute(int hamster_index) {
	int leftProximity = hamster_read(HAMSTER_LEFT_PROXIMITY);
	int rightProximity = hamster_read(HAMSTER_RIGHT_PROXIMITY);

	// left wheel
	if(leftProximity > 15)
		hamster_write(HAMSTER_LEFT_WHEEL, (40 - leftProximity) * 4);
	else
		hamster_write(HAMSTER_LEFT_WHEEL, 0);

	// right wheel
	if(rightProximity > 15)
		hamster_write(HAMSTER_RIGHT_WHEEL, (40 - rightProximity) * 4);
	else
		hamster_write(HAMSTER_RIGHT_WHEEL, 0);
}

void example08(void) { // simple hand follower
	hamster_create();
	hamster_set_executable(hand_execute);

	// 10 seconds
	hamster_wait(10000);

	hamster_dispose_all();
}

double note1 = 0, note2 = 0;
void theremin_execute1(int hamster_index) {
	int proximity = hamster_read_multi(hamster_index, HAMSTER_LEFT_PROXIMITY);
	if(proximity < 10) proximity = 0;
	note1 = (note1 * 9 + proximity) / 10;
	hamster_write_multi(hamster_index, HAMSTER_NOTE, (int)note1);
}

void theremin_execute2(int hamster_index) {
	int proximity = hamster_read_multi(hamster_index, HAMSTER_LEFT_PROXIMITY);
	if(proximity < 10) proximity = 0;
	note2 = (note2 * 9 + proximity) / 10;
	hamster_write_multi(hamster_index, HAMSTER_NOTE, (int)note2);
}

void example09(void) { // dual theremin sound
	note1 = 0;
	note2 = 0;

	hamster_create_multi(2);
	hamster_set_executable_multi(0, theremin_execute1);
	hamster_set_executable_multi(1, theremin_execute2);

	// 10 seconds
	hamster_wait(10000);

	hamster_dispose_all();
}

void example10(void) { // mutiple basic movement
	int i, j;

	hamster_create_multi(2);

	for(i = 0; i < 10; ++i) {
		for(j = 0; j < 2; ++j) {
			hamster_write_multi(j, HAMSTER_LEFT_WHEEL, 50);
			hamster_write_multi(j, HAMSTER_RIGHT_WHEEL, 50);
		}
		hamster_wait(500);

		for(j = 0; j < 2; ++j) {
			hamster_write_multi(j, HAMSTER_LEFT_WHEEL, -50);
			hamster_write_multi(j, HAMSTER_RIGHT_WHEEL, -50);
		}
		hamster_wait(500);

		for(j = 0; j < 2; ++j) {
			hamster_write_multi(j, HAMSTER_LEFT_WHEEL, -50);
			hamster_write_multi(j, HAMSTER_RIGHT_WHEEL, 50);
		}
		hamster_wait(500);
	}

	hamster_dispose_all();
}

void hand_execute_multi(int hamster_index) {
	int leftProximity = hamster_read_multi(hamster_index, HAMSTER_LEFT_PROXIMITY);
	int rightProximity = hamster_read_multi(hamster_index, HAMSTER_RIGHT_PROXIMITY);

	// left wheel
	if(leftProximity > 15)
		hamster_write_multi(hamster_index, HAMSTER_LEFT_WHEEL, (40 - leftProximity) * 4);
	else
		hamster_write_multi(hamster_index, HAMSTER_LEFT_WHEEL, 0);

	// right wheel
	if(rightProximity > 15)
		hamster_write_multi(hamster_index, HAMSTER_RIGHT_WHEEL, (40 - rightProximity) * 4);
	else
		hamster_write_multi(hamster_index, HAMSTER_RIGHT_WHEEL, 0);
}

void example11(void) { // multiple hand follower
	int i;
	
	hamster_create_multi(2);

	for(i = 0; i < 2; ++i) {
		hamster_set_executable_multi(i, hand_execute_multi);
	}

	// 10 seconds
	hamster_wait(10000);

	hamster_dispose_all();
}

int main(int argc, char *argv[]) {
	example01(); // simple movement
	//example02(); // acceleration
	//example03(); // led
	//example04(); // basic movement
	//example05(); // staggering walk
	//example06(); // siren
	//example07(); // theremin sound
	//example08(); // simple hand follower
	//example09(); // dual theremin sound
	//example10(); // mutiple basic movement
	//example11(); // multiple hand follower
	
	return 0;
}
